# -*- coding:utf-8 -*-
# author:jackwu 
# time:2022/5/26
# description: 
import os
import sys

PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(PROJECT_PATH)
